# -*- encoding: utf-8 -*-
#
# Access data from techshop.ws
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#


from classes import Lab

import json
from bs4 import BeautifulSoup
import requests
from geojson import dumps, Feature, Point, FeatureCollection
from geopy.geocoders import Nominatim
import pandas as pd


# Geocoding variable
geolocator = Nominatim()

# Endpoints
techshop_us_url = "http://techshop.ws/locations.html"
techshop_global_url = "http://techshop.ws/ts_global.html"


class Techshop(Lab):
    """Represents a Techshop as it is described on techshop.ws."""

    def __init__(self):
        self.source = "techshop.ws"
        self.lab_type = "Techshop"


def data_from_techshop_ws(tws_url):
    """Scrapes data from techshop.ws."""

    r = requests.get(tws_url)
    if r.status_code == 200:
        data = BeautifulSoup(r.text, "lxml")
    else:
        data = "There was an error while accessing data on techshop.ws."

    return data


def get_labs(format):
    """Gets Techshop data from techshop.ws."""

    techshops_soup = data_from_techshop_ws(techshop_us_url)
    techshops = {}

    # Load all the TechShops
    # By first parsing the html

    data = techshops_soup.findAll('div', attrs={'id': 'main-content'})
    for element in data:
        links = element.findAll('a')
        hrefs = {}
        for k, a in enumerate(links):
            if "contact" not in a['href']:
                hrefs[k] = a['href']
        for k, v in hrefs.iteritems():
            if "http://techshop.ws/" not in v:
                hrefs[k] = "http://techshop.ws/" + v
            else:
                hrefs[k] = v
        for k, v in hrefs.iteritems():
            if "http://techshop.com/" in v:
                hrefs[k] = v.replace("http://techshop.com/", "")

    # Remove duplicate pages
    hr = []
    for key, value in hrefs.iteritems():
        if value not in hr:
            hr.append(value)
    hrefs = hr

    # Check all pages
    for page in hrefs:
        data = data_from_techshop_ws(page)
        current_lab = Techshop()
        name = data.title.contents[0].split('-- ')[1].encode('utf-8')
        if "TechShop" not in name:
            name = "TechShop " + name
        current_lab.name = name
        current_lab.slug = name
        current_lab.url = page
        # Find Facebook and Twitter links
        current_lab.links = {"facebook": "", "twitter": ""}
        page_links = data.findAll('a')
        for link in page_links:
            if link.has_attr("href"):
                if "facebook" in link.attrs["href"]:
                    current_lab.links["facebook"] = link.attrs["href"]
                if "twitter" in link.attrs["href"]:
                    current_lab.links["twitter"] = link.attrs["href"]
        # Find the coordinates by analysing the embedded google map
        iframes = data.findAll('iframe')
        if len(iframes) != 0:
            for iframe in iframes:
                embed_url = iframe.attrs["src"]
                if "google" in embed_url:
                    two_d = embed_url.find("2d")
                    three_d = embed_url.find("3d")
                    longitude = embed_url[two_d:].split('!')[0]
                    latitude = embed_url[three_d:].split('!')[0]
                    longitude = longitude[2:]
                    latitude = latitude[2:]
        # ... or the link to google map
        else:
            page_links = data.findAll('a')
            for link in page_links:
                # one case...
                if "maps.google.com/" in link.attrs["href"]:
                    embed_url = link.attrs["href"]
                    if "ll=" in embed_url:
                        first_string = embed_url.split('&sspn')[0]
                        coordinates = first_string.split('ll=')[1]
                        latitude = coordinates.split(',')[0]
                        longitude = coordinates.split(',')[1]
                # ... another case
                elif "www.google.com/maps" in link.attrs["href"]:
                    embed_url = link.attrs["href"]
                    if "1d" in embed_url:
                        one_d = embed_url.find("1d")
                        two_d = embed_url.find("2d")
                        longitude = embed_url[one_d:].split('!')[0]
                        latitude = embed_url[two_d:].split('!')[0]
                        longitude = longitude[2:]
                        latitude = latitude[2:]
        current_lab.latitude = latitude
        current_lab.longitude = longitude
        current_lab.continent = "North America"
        current_lab.country_code = "USA"
        current_lab.country = "United States of America"
        location = geolocator.reverse((latitude, longitude))
        if "city" in location.raw["address"]:
            current_lab.county = location.raw["address"]["city"].encode(
                'utf-8')
        if "county" in location.raw["address"]:
            current_lab.county = location.raw["address"]["county"].encode(
                'utf-8')
        if "state" in location.raw["address"]:
            current_lab.state = location.raw["address"]["state"].encode(
                'utf-8')
        if "postcode" in location.raw["address"]:
            current_lab.postal_code = location.raw["address"][
                "postcode"].encode('utf-8')
        current_lab.address_1 = location.address.encode('utf-8')

        # Add the lab to the list
        techshops[current_lab.slug] = current_lab

    # Return a dictiornary / json
    if format.lower() == "dict" or format.lower() == "json":
        output = {}
        for j in techshops:
            output[j] = techshops[j].__dict__
    # Return a geojson
    elif format.lower() == "geojson" or format.lower() == "geo":
        labs_list = []
        for l in techshops:
            single = techshops[l].__dict__
            single_lab = Feature(
                type="Feature",
                geometry=Point((single["latitude"], single["longitude"])),
                properties=single)
            labs_list.append(single_lab)
        output = dumps(FeatureCollection(labs_list))
    # Return a Pandas DataFrame
    elif format.lower() == "pandas" or format.lower() == "dataframe":
        output = {}
        for j in techshops:
            output[j] = techshops[j].__dict__
        # Transform the dict into a Pandas DataFrame
        output = pd.DataFrame.from_dict(output)
        output = output.transpose()
    # Return an object
    elif format.lower() == "object" or format.lower() == "obj":
        output = techshops
    # Default: return an oject
    else:
        output = techshops
    # Return a proper json
    if format.lower() == "json":
        output = json.dumps(output)
    return output


def labs_count():
    """Gets the number of current Techshops listed on techshops.ws."""

    techshops = get_labs("object")

    return len(techshops)


if __name__ == "__main__":
    pass



# -*- encoding: utf-8 -*-
#
# Access data from makeinitaly.foundation
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#


from classes import Lab

import json
from simplemediawiki import MediaWiki
import pandas as pd


makeinitaly__foundation_api_url = "http://makeinitaly.foundation/wiki/api.php"


class MILab(Lab):

    """Represents a Lab as it is described on makeinitaly.foundation."""

    def __init__(self):
        self.source = "makeinitaly.foundation"
        self.lab_type = "Lab on makeinitaly.foundation"
        self.continent = "Europe"
        self.country_code = "IT"
        self.country = "Italy"


def get_lab_text(lab_slug, language):
    """Gets text description in English or Italian from a single lab from makeinitaly.foundation."""
    if language == "English" or language == "english" or language == "EN" or language == "En":
        language = "en"
    elif language == "Italian" or language == "italian" or language == "IT" or language == "It" or language == "it":
        language = "it"
    else:
        language = "en"
    wiki = MediaWiki(makeinitaly__foundation_api_url)
    wiki_response = wiki.call(
        {'action': 'query',
         'titles': lab_slug + "/" + language,
         'prop': 'revisions',
         'rvprop': 'content'})

    # If we don't know the pageid...
    for i in wiki_response["query"]["pages"]:
        if "revisions" in wiki_response["query"]["pages"][i]:
            content = wiki_response["query"]["pages"][i]["revisions"][0]["*"]
        else:
            content = ""

    # Clean the resulting string/list
    newstr01 = content.replace("}}", "")
    newstr02 = newstr01.replace("{{", "")
    result = newstr02.rstrip("\n|").split("\n|")

    return result[0]


def get_single_lab(lab_slug):
    """Gets data from a single lab from makeinitaly.foundation."""
    wiki = MediaWiki(makeinitaly__foundation_api_url)
    wiki_response = wiki.call(
        {'action': 'query',
         'titles': lab_slug,
         'prop': 'revisions',
         'rvprop': 'content'})

    # If we don't know the pageid...
    for i in wiki_response["query"]["pages"]:
        content = wiki_response["query"]["pages"][i]["revisions"][0]["*"]

    # Clean the resulting string/list
    newstr01 = content.replace("}}", "")
    newstr02 = newstr01.replace("{{", "")
    result = newstr02.rstrip("\n|").split("\n|")
    # result.remove(u'FabLab')

    # Transform the data into a Lab object
    current_lab = MILab()

    # Add existing data
    for i in result:
        if "coordinates=" in i:
            value = i.replace("coordinates=", "")
            current_lab.coordinates = value
            latlong = []
            if ", " in value:
                latlong = value.rstrip(", ").split(", ")
            elif " , " in value:
                latlong = value.rstrip(" , ").split(" , ")
            else:
                latlong = ["", ""]
            current_lab.latitude = latlong[0]
            current_lab.longitude = latlong[1]
        elif "province=" in i:
            value = i.replace("province=", "")
            current_lab.province = value.upper()
        elif "region=" in i:
            value = i.replace("region=", "")
            current_lab.region = value
        elif "address=" in i:
            value = i.replace("address=", "")
            current_lab.address = value
        elif "city=" in i:
            value = i.replace("city=", "")
            current_lab.city = value
        elif "fablabsio=" in i:
            value = i.replace("fablabsio=", "")
            current_lab.fablabsio = value
        elif "website=" in i:
            value = i.replace("website=", "")
            current_lab.website = value
        elif "facebook=" in i:
            value = i.replace("facebook=", "")
            current_lab.facebook = value
        elif "twitter=" in i:
            value = i.replace("twitter=", "")
            current_lab.twitter = value
        elif "email=" in i:
            value = i.replace("email=", "")
            current_lab.email = value
        elif "manager=" in i:
            value = i.replace("manager=", "")
            current_lab.manager = value
        elif "birthyear=" in i:
            value = i.replace("birthyear=", "")
            current_lab.birthyear = value

    current_lab.text_en = get_lab_text(lab_slug=lab_slug, language="en")
    current_lab.text_it = get_lab_text(lab_slug=lab_slug, language="it")

    return current_lab


def get_labs(format):
    """Gets data from all labs from makeinitaly.foundation."""

    labs = []

    # Get the first page of data
    wiki = MediaWiki(makeinitaly__foundation_api_url)
    wiki_response = wiki.call(
        {'action': 'query',
         'list': 'categorymembers',
         'cmtitle': 'Category:Italian_FabLabs',
         'cmlimit': '500'})
    if "query-continue" in wiki_response:
        nextpage = wiki_response[
            "query-continue"]["categorymembers"]["cmcontinue"]

    urls = []
    for i in wiki_response["query"]["categorymembers"]:
        urls.append(i["title"].replace(" ", "_"))

    # Load all the Labs in the first page
    for i in urls:
        current_lab = get_single_lab(i)
        labs.append(current_lab)

    # Load all the Labs from the other pages
    while "query-continue" in wiki_response:
        wiki = MediaWiki(makeinitaly__foundation_api_url)
        wiki_response = wiki.call({'action': 'query',
                                   'list': 'categorymembers',
                                   'cmtitle': 'Category:Hackerspace',
                                   'cmlimit': '500',
                                   "cmcontinue": nextpage})

        urls = []
        for i in wiki_response["query"]["categorymembers"]:
            urls.append(i["title"].replace(" ", "_"))

        # Load all the Labs
        for i in urls:
            current_lab = get_single_lab(i, data_format)
            labs.append(current_lab)

        if "query-continue" in wiki_response:
            nextpage = wiki_response[
                "query-continue"]["categorymembers"]["cmcontinue"]
        else:
            break

    # Transform the list into a dictionary
    labs_dict = {}
    for j, k in enumerate(labs):
        labs_dict[j] = k.__dict__

    # Return a dictiornary / json
    if format.lower() == "dict" or format.lower() == "json":
        output = labs_dict
    # Return a geojson
    elif format.lower() == "geojson" or format.lower() == "geo":
        labs_list = []
        for l in labs_dict:
            single = labs_dict[l].__dict__
            single_lab = Feature(
                type="Feature",
                geometry=Point((single["latitude"], single["longitude"])),
                properties=single)
            labs_list.append(single_lab)
        output = dumps(FeatureCollection(labs_list))
    # Return a Pandas DataFrame
    elif format.lower() == "pandas" or format.lower() == "dataframe":
        output = {}
        for j in labs_dict:
            output[j] = labs_dict[j].__dict__
        # Transform the dict into a Pandas DataFrame
        output = pd.DataFrame.from_dict(output)
        output = output.transpose()
    # Return an object
    elif format.lower() == "object" or format.lower() == "obj":
        output = labs
    # Default: return an object
    else:
        output = labs
    # Return a proper json
    if format.lower() == "json":
        output = json.dumps(labs_dict)
    return output


def labs_count():
    """Gets the number of current Labs registered on makeinitaly.foundation."""

    labs = get_labs(data_format="dict")

    return len(labs)


if __name__ == "__main__":
    pass



# -*- encoding: utf-8 -*-
#
# Access data from hackaday.io
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#


from classes import Lab

import json
import requests
from geojson import dumps, Feature, Point, FeatureCollection
from geopy.geocoders import Nominatim
import pandas as pd

# Geocoding variable
geolocator = Nominatim()


# Endpoints
# The documented endpoint does not have coordinates,
# the undocumented one has them, so for the moment we use the latter.
# The undocumented endpoint does not need API keys or OAuth.
# hackaday.io API documentation:
# https://dev.hackaday.io/doc/api/get-pages
# Register your app for the API key here:
# https://dev.hackaday.io/applications
client_id = "..."
client_secret = "..."
API_key = "..."
# Documented endpoint for the list of hackerspaces
hackaday_io_labs_api_url = "https://api.hackaday.io/v1/pages/hackerspaces?api_key=" + API_key
# Undocumented endpoint for the map of hackerspaces
hackaday_io_labs_map_url = "http://hackaday.io/api/location/hackerspaces"


class Hackerspace(Lab):
    """Represents a Hackerspace as it is described on hackaday.io."""

    def __init__(self):
        self.source = "hackaday.io"
        self.lab_type = "Hackerspace"


def data_from_hackaday_io(endpoint):
    """Gets data from hackaday.io."""

    data = requests.get(endpoint).json()

    return data


def get_labs(format):
    """Gets Hackerspaces data from hackaday.io."""

    hackerspaces_json = data_from_hackaday_io(hackaday_io_labs_map_url)
    hackerspaces = {}

    # Load all the Hackerspaces
    for i in hackerspaces_json:
        current_lab = Hackerspace()
        current_lab.id = i["id"]
        current_lab.url = "https://hackaday.io/hackerspace/" + current_lab.id
        current_lab.name = i["name"]
        if len(i["description"]) != 0:
            current_lab.description = i["description"]
        elif len(i["summary"]) != 0:
            current_lab.description = i["summary"]
        current_lab.created_at = i["moments"]["exact"]

        # Check if there are coordinates
        if i["latlon"] is not None:
            latlon = json.loads(i["latlon"])
            current_lab.latitude = latlon["lat"]
            current_lab.longitude = latlon["lng"]
            # Get country, county and city from them
            country = geolocator.reverse(
                [latlon["lat"], latlon["lng"]])
            current_lab.country = country.raw[
                "address"]["country"]
            current_lab.address = country.raw["display_name"]
            current_lab.address_1 = country.raw["display_name"]
            current_lab.country_code = country.raw[
                "address"]["country_code"]
            current_lab.county = country.raw[
                "address"]["state_district"]
            current_lab.city = country.raw[
                "address"]["city"]
            current_lab.postal_code = country.raw[
                "address"]["postcode"]
        else:
            # For labs without a location or coordinates
            # add 0,0 as coordinates
            current_lab.latitude = 0.0
            current_lab.longitude = 0.0

        # Add the lab
        hackerspaces[i["name"]] = current_lab

    # Return a dictiornary / json
    if format.lower() == "dict" or format.lower() == "json":
        output = {}
        for j in hackerspaces:
            output[j] = hackerspaces[j].__dict__
    # Return a geojson
    elif format.lower() == "geojson" or format.lower() == "geo":
        labs_list = []
        for l in hackerspaces:
            single = hackerspaces[l].__dict__
            single_lab = Feature(
                type="Feature",
                geometry=Point((single["latitude"], single["longitude"])),
                properties=single)
            labs_list.append(single_lab)
        output = dumps(FeatureCollection(labs_list))
    # Return a Pandas DataFrame
    elif format.lower() == "pandas" or format.lower() == "dataframe":
        output = {}
        for j in hackerspaces:
            output[j] = hackerspaces[j].__dict__
        # Transform the dict into a Pandas DataFrame
        output = pd.DataFrame.from_dict(output)
        output = output.transpose()
    # Return an object
    elif format.lower() == "object" or format.lower() == "obj":
        output = hackerspaces
    # Default: return an oject
    else:
        output = hackerspaces
    # Return a proper json
    if format.lower() == "json":
        output = json.dumps(output)
    return output


def labs_count():
    """Gets the number of current Hackerspaces listed on hackaday.io."""

    hackerspaces = data_from_hackaday_io(hackaday_io_labs_api_url)

    return len(hackerspaces["labs"])


if __name__ == "__main__":
    pass



# -*- encoding: utf-8 -*-
#
# Access data from NESTA at https://github.com/nesta-uk/UK-makerspaces
# Data license: CC Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#


from classes import Lab

import json
import pandas as pd
from geojson import dumps, Feature, Point, FeatureCollection
from geopy.geocoders import Nominatim
import pandas as pd

# Geocoding variable
geolocator = Nominatim()

# Endpoints
nesta_uk_url = "https://raw.githubusercontent.com/nesta-uk/UK-makerspaces/master/ukmakerspacesidentifiabledata.csv"


class UKMakerspace(Lab):
    """Represents a UK Makerspace from the NESTA research, in a simplified way."""

    def __init__(self):
        self.source = "NESTA"
        self.lab_type = "UK Makerspace from the NESTA"
        self.continent = "Europe"
        self.country_code = "UK"
        self.country = "United Kingdom"


def data_from_nesta():
    """Read data from the GitHub repo."""

    data = pd.read_csv(nesta_uk_url)

    return data


def get_labs(format):
    """Gets current UK Makerspaces data as listed by NESTA."""

    ukmakerspaces_data = data_from_nesta()
    ukmakerspaces = {}

    # Iterate over csv rows
    for index, row in ukmakerspaces_data.iterrows():
        current_lab = UKMakerspace()
        current_lab.address_1 = row["Address"].replace("\r", " ")
        current_lab.address_2 = row["Region"].replace("\r", " ") + " - " + row["Area"].replace("\r", " ")
        current_lab.city = ""
        current_lab.county = ""
        current_lab.email = row["Email address"]
        current_lab.latitude = ""
        current_lab.longitude = ""
        current_lab.links = ""
        current_lab.name = row["Name of makerspace"]
        current_lab.phone = row["Phone number"]
        current_lab.postal_code = row["Postcode"]
        current_lab.url = row["Website / URL"]

        # Add the lab, with a slug from the name
        ukmakerspaces[current_lab.name] = current_lab

    # Return a dictiornary / json
    if format.lower() == "dict" or format.lower() == "json":
        output = {}
        for j in ukmakerspaces:
            output[j] = ukmakerspaces[j].__dict__
    # Return a geojson
    elif format.lower() == "geojson" or format.lower() == "geo":
        labs_list = []
        for l in ukmakerspaces:
            single = ukmakerspaces[l].__dict__
            single_lab = Feature(
                type="Feature",
                geometry=Point((single["latitude"], single["longitude"])),
                properties=single)
            labs_list.append(single_lab)
        output = dumps(FeatureCollection(labs_list))
    # Return a Pandas DataFrame
    elif format.lower() == "pandas" or format.lower() == "dataframe":
        output = {}
        for j in ukmakerspaces:
            output[j] = ukmakerspaces[j].__dict__
        # Transform the dict into a Pandas DataFrame
        output = pd.DataFrame.from_dict(output)
        output = output.transpose()
    # Return an object
    elif format.lower() == "object" or format.lower() == "obj":
        output = ukmakerspaces
    # Default: return an oject
    else:
        output = ukmakerspaces
    # Return a proper json
    if format.lower() == "json":
        output = json.dumps(output)
    return output


def labs_count():
    """Gets the number of current UK Makerspaces listed by NESTA."""

    ukmakerspaces = get_labs("object")

    return len(ukmakerspaces)


if __name__ == "__main__":
    pass



# -*- encoding: utf-8 -*-
#
# Access data from makery.info
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#


from classes import Lab

import json
import requests
from geojson import dumps, Feature, Point, FeatureCollection
from geopy.geocoders import Nominatim
import pandas as pd


# Geocoding variable
geolocator = Nominatim()

# Endpoints
makery_info_labs_api_url = "http://www.makery.info/api/labs/"


class MakeryLab(Lab):
    """Represents a Lab as it is described on makery.info."""

    def __init__(self):
        self.source = "makery.info"
        self.lab_type = "Lab on makery.info"


def data_from_makery_info(endpoint):
    """Gets data from makery.info."""

    data = requests.get(endpoint).json()

    return data


def get_labs(format):
    """Gets Lab data from makery.info."""

    labs_json = data_from_makery_info(makery_info_labs_api_url)
    labs = {}

    # Load all the FabLabs
    for i in labs_json["labs"]:
        current_lab = MakeryLab()
        current_lab.address_1 = i["address_1"]
        current_lab.address_2 = i["address_2"]
        current_lab.address_notes = i["address_notes"]
        current_lab.avatar = i["avatar"]
        current_lab.blurb = i["blurb"]
        current_lab.capabilities = i["capabilities"]
        current_lab.city = i["city"]
        current_lab.country_code = i["country_code"]
        current_lab.county = i["county"]
        current_lab.description = i["description"]
        current_lab.email = i["email"]
        current_lab.header_image_src = i["header_image_src"]
        current_lab.id = i["id"]
        current_lab.kind_name = i["kind_name"]
        # Some labs do not have coordinates
        if i["latitude"] is None or i["longitude"] is None:
            address = i["address_1"] + i["city"] + i["country_code"]
            try:
                location = geolocator.geocode(address)
                current_lab.latitude = location.latitude
                current_lab.longitude = location.longitude
            except:
                try:
                    location = geolocator.geocode(i["city"])
                    current_lab.latitude = location.latitude
                    current_lab.longitude = location.longitude
                except:
                    # For labs without a city, add 0,0 as coordinates
                    current_lab.latitude = 0.0
                    current_lab.longitude = 0.0
        else:
            current_lab.latitude = i["latitude"]
            current_lab.longitude = i["longitude"]
        current_lab.links = i["links"]
        current_lab.name = i["name"]
        current_lab.parent_id = i["parent_id"]
        current_lab.phone = i["phone"]
        current_lab.postal_code = i["postal_code"]
        current_lab.slug = i["slug"]
        current_lab.url = i["url"]
        # Add the lab
        labs[i["slug"]] = current_lab

    # Return a dictiornary / json
    if format.lower() == "dict" or format.lower() == "json":
        output = {}
        for j in labs:
            output[j] = labs[j].__dict__
    # Return a geojson
    elif format.lower() == "geojson" or format.lower() == "geo":
        labs_list = []
        for l in labs:
            single = labs[l].__dict__
            single_lab = Feature(
                type="Feature",
                geometry=Point((single["latitude"], single["longitude"])),
                properties=single)
            labs_list.append(single_lab)
        output = dumps(FeatureCollection(labs_list))
    # Return a Pandas DataFrame
    elif format.lower() == "pandas" or format.lower() == "dataframe":
        output = {}
        for j in labs_list:
            output[j] = labs_list[j].__dict__
        # Transform the dict into a Pandas DataFrame
        output = pd.DataFrame.from_dict(output)
        output = output.transpose()
    # Return an object
    elif format.lower() == "object" or format.lower() == "obj":
        output = labs
    # Default: return an oject
    else:
        output = labs
    # Return a proper json
    if format.lower() == "json":
        output = json.dumps(output)
    return output


def labs_count():
    """Gets the number of current Labs listed on makery.info."""

    labs = data_from_makery_info(makery_info_labs_api_url)

    return len(labs["labs"])


if __name__ == "__main__":
    pass



# -*- encoding: utf-8 -*-
#
# Access data from diybio.org
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#


from classes import Lab
from utils import get_location

import json
from bs4 import BeautifulSoup
import requests
from geojson import dumps, Feature, Point, FeatureCollection
import us
import pandas as pd

# Endpoints
diy_bio_labs_url = "https://diybio.org/local/"


class DiyBioLab(Lab):
    """Represents a DIYBio Lab as it is described on diybio.org."""

    def __init__(self):
        self.source = "diybio.org"
        self.lab_type = "DIYBio Lab"


def data_from_diybio_org():
    """Scrapes data from diybio.org."""

    r = requests.get(diy_bio_labs_url)

    if r.status_code == 200:
        # Fix a problem in the html source while loading it
        data = BeautifulSoup(r.text.replace(u'\xa0', u''), "lxml")
    else:
        data = "There was an error while accessing data on diybio.org."

    return data


def get_labs(format, open_cage_api_key):
    """Gets DIYBio Lab data from diybio.org."""

    diybiolabs_soup = data_from_diybio_org()
    diybiolabs = {}

    rows_list = []
    continents_dict = {}
    continents_order = 0
    ranges_starting_points = []

    # Load all the DIYBio Labs
    # By first parsing the html

    # Parse table rows
    for row in diybiolabs_soup.select("table tr"):
        cells = row.find_all('td')
        rows_list.append(cells)

    # Find the continents in order to iterate over their children td
    for k, row in enumerate(rows_list):
        for col in row:
            if col.find('h3'):
                for h3 in col.findAll('h3'):
                    ranges_starting_points.append(k)
                    continents_dict[continents_order] = h3.get_text()
                    continents_order += 1

    # Find the rows of each continent
    ranges = {}
    for k, j in enumerate(reversed(ranges_starting_points)):
        if k < len(ranges_starting_points) - 1:
            ranges[k] = {"start": ranges_starting_points[k],
                         "end": ranges_starting_points[k + 1]}
        else:
            # The last continent, Oceania
            ranges[k] = {"start": ranges_starting_points[k],
                         "end": len(rows_list)}

    # Iterate over the range of each continent to find the Labs
    for i in ranges:
        # The +1 just avoids the H3 line
        for j in range(ranges[i]["start"] + 1, ranges[i]["end"]):
            # Avoid empty rows by measuring the lenght of the content of each cell and with a boolean check
            rules = [len(n) == 0 for n in rows_list[j]]
            if False in rules:
                current_lab = DiyBioLab()
                current_lab.city = rows_list[j][1].contents[0].encode('utf-8')
                # Data from the USA is not really well formatted
                if continents_dict[i] == "USA-EAST" or continents_dict[
                        i] == "USA-WEST":
                    current_lab.state = rows_list[j][2].contents[0].replace(
                        " ", "").encode('utf-8')
                else:
                    current_lab.country_code = rows_list[j][2].contents[
                        0].encode('utf-8')
                current_lab.url = rows_list[j][3].contents[0].attrs['href']
                # Each lab is identified by the simplified url
                slug = current_lab.url
                if "http://" in slug:
                    slug = slug.replace("http://", "")
                elif "https://" in slug:
                    slug = slug.replace("https://", "")
                if "www." in slug:
                    slug = slug.replace("www.", "")
                current_lab.name = slug
                current_lab.slug = slug

                # Data from the USA is not really well formatted
                if continents_dict[i] == "USA-EAST" or continents_dict[i] == "USA-WEST":
                    current_lab.continent = "North America"
                    current_lab.country_code = "USA"
                    current_lab.country = "United States of America"
                    current_lab.state = us.states.lookup(
                        current_lab.state).name

                # Get address from city
                address = get_location(query=current_lab.city, format="direct", api_key=open_cage_api_key)
                current_lab.continent = address["continent"]
                current_lab.latitude = address["latitude"]
                current_lab.longitude = address["longitude"]
                current_lab.address_1 = address["address_1"]
                current_lab.country = address["country"]
                current_lab.country_code = address["country_code"]
                current_lab.latitude = address["latitude"]
                current_lab.longitude = address["longitude"]
                current_lab.county = address["county"]
                current_lab.postal_code = address["postal_code"]
                current_lab.state = address["state"]

                # Add the lab to the list
                diybiolabs[slug] = current_lab
                del current_lab

    # Return a dictionary / json
    if format.lower() == "dict":
        output = {}
        for j in diybiolabs:
            output[j] = diybiolabs[j].__dict__
    # Return a geojson
    elif format.lower() == "geojson" or format.lower() == "geo":
        labs_list = []
        for l in diybiolabs:
            single = diybiolabs[l].__dict__
            single_lab = Feature(
                type="Feature",
                geometry=Point((single["latitude"], single["longitude"])),
                properties=single)
            labs_list.append(single_lab)
        output = dumps(FeatureCollection(labs_list))
    # Return a Pandas DataFrame
    elif format.lower() == "pandas" or format.lower() == "dataframe":
        output = {}
        for j in diybiolabs:
            output[j] = diybiolabs[j].__dict__
        # Transform the dict into a Pandas DataFrame
        output = pd.DataFrame.from_dict(output)
        # Put labs names as the index, to make it coherent with other APIs
        output = output.transpose()
    # Return an object
    elif format.lower() == "object" or format.lower() == "obj":
        output = diybiolabs
    # Default: return an oject
    else:
        output = diybiolabs
    # Return a proper json
    if format.lower() == "json":
        output = json.dumps(diybiolabs)
    return output


def labs_count():
    """Gets the number of current DIYBio Labs listed on diybio.org."""

    diybiolabs = get_labs("object")

    return len(diybiolabs)


if __name__ == "__main__":
    pass



# -*- encoding: utf-8 -*-
#
# Access data from fablabs.io
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#


from classes import Lab

import json
import requests
from geojson import dumps, Feature, Point, FeatureCollection
from geopy.geocoders import Nominatim
import pycountry
from pycountry_convert import convert_country_alpha2_to_continent
from time import sleep
import pandas as pd


# Geocoding variable
geolocator = Nominatim()

# Endpoints
fablabs_io_labs_api_url_v0 = "https://api.fablabs.io/v0/labs.json"
fablabs_io_projects_api_url_v0 = "https://api.fablabs.io/v0/projects.json"


class FabLab(Lab):
    """Represents a Fab Lab as it is described on fablabs.io."""

    def __init__(self):
        self.source = "fablabs.io"
        self.lab_type = "Fab Lab"


class Project(object):
    """Represents a project as it is described on fablabs.io."""

    def __init__(self):
        self.id = ""
        self.title = ""
        self.description = ""
        self.github = ""
        self.web = ""
        self.dropbox = ""
        self.bitbucket = ""
        self.lab_id = ""
        self.lab = ""
        self.owner_id = ""
        self.created_at = ""
        self.updated_at = ""
        self.vimeo = ""
        self.flickr = ""
        self.youtube = ""
        self.drive = ""
        self.twitter = ""
        self.facebook = ""
        self.googleplus = ""
        self.instagram = ""
        self.status = ""
        self.version = ""
        self.faq = ""
        self.scope = ""
        self.community = ""
        self.lookingfor = ""
        self.cover = ""
        self.type = "Project in a Fab Lab"


def data_from_fablabs_io(endpoint):
    """Gets data from fablabs.io."""

    data = requests.get(endpoint).json()

    return data


def get_labs(format):
    """Gets Fab Lab data from fablabs.io."""

    fablabs_json = data_from_fablabs_io(fablabs_io_labs_api_url_v0)
    fablabs = {}

    # Load all the FabLabs
    for i in fablabs_json["labs"]:
        current_lab = FabLab()
        current_lab.name = i["name"]
        current_lab.address_1 = i["address_1"]
        current_lab.address_2 = i["address_2"]
        current_lab.address_notes = i["address_notes"]
        current_lab.avatar = i["avatar_url"]
        current_lab.blurb = i["blurb"]
        current_lab.capabilities = i["capabilities"]
        if i["city"].isupper():
            i["city"] = i["city"].title()
        current_lab.city = i["city"]
        current_lab.country_code = i["country_code"]
        current_lab.county = i["county"]
        current_lab.description = i["description"]
        current_lab.email = i["email"]
        current_lab.id = i["id"]
        current_lab.phone = i["phone"]
        current_lab.postal_code = i["postal_code"]
        current_lab.slug = i["slug"]
        current_lab.url = i["url"]

        current_lab.continent = convert_country_alpha2_to_continent(i["country_code"].upper())
        current_country = pycountry.countries.get(alpha_2=i["country_code"].upper())
        current_lab.country_code = current_country.alpha_3
        current_lab.country = current_country.name

        # Check coordinates
        if i["longitude"] is not None:
            current_lab.longitude = i["longitude"]
        else:
            current_lab.longitude = 0.0
        if i["latitude"] is not None:
            current_lab.latitude = i["latitude"]
        else:
            current_lab.latitude = 0.0

        # Find Facebook and Twitter links, add also the other ones
        current_lab.links = {"facebook": "", "twitter": ""}
        for link in i["links"]:
            if "facebook" in link["url"]:
                current_lab.links["facebook"] = link["url"]
            elif "twitter" in link["url"]:
                current_lab.links["twitter"] = link["url"]
            else:
                current_lab.links[link["id"]] = link["url"]

        # Add the lab to the list
        fablabs[i["slug"]] = current_lab

    # Return a dictiornary / json
    if format.lower() == "dict" or format.lower() == "json":
        output = {}
        for j in fablabs:
            output[j] = fablabs[j].__dict__
    # Return a geojson
    elif format.lower() == "geojson" or format.lower() == "geo":
        labs_list = []
        for l in fablabs:
            single = fablabs[l].__dict__
            single_lab = Feature(
                type="Feature",
                geometry=Point((single["latitude"], single["longitude"])),
                properties=single)
            labs_list.append(single_lab)
        output = dumps(FeatureCollection(labs_list))
    # Return a Pandas DataFrame
    elif format.lower() == "pandas" or format.lower() == "dataframe":
        output = {}
        for j in fablabs:
            output[j] = fablabs[j].__dict__
        # Transform the dict into a Pandas DataFrame
        output = pd.DataFrame.from_dict(output)
        output = output.transpose()
    # Return an object
    elif format.lower() == "object" or format.lower() == "obj":
        output = fablabs
    # Default: return an oject
    else:
        output = fablabs
    # Return a proper json
    if format.lower() == "json":
        output = json.dumps(output)
    return output


def labs_count():
    """Gets the number of current Fab Labs registered on fablabs.io."""

    fablabs = data_from_fablabs_io(fablabs_io_labs_api_url_v0)

    return len(fablabs["labs"])


def get_projects(format):
    """Gets projects data from fablabs.io."""

    projects_json = data_from_fablabs_io(fablabs_io_projects_api_url_v0)
    projects = {}
    project_url = "https://www.fablabs.io/projects/"
    fablabs = get_labs(format="object")

    # Load all the FabLabs
    for i in projects_json["projects"]:
        i = i["projects"]
        current_project = Project()
        current_project.id = i["id"]
        current_project.title = i["title"]
        current_project.description = i["description"]
        current_project.github = i["github"]
        current_project.web = i["web"]
        current_project.dropbox = i["dropbox"]
        current_project.bitbucket = i["bitbucket"]
        current_project.lab_id = i["lab_id"]
        # Add the lab of the project
        if i["lab_id"] is not None:
            for k in fablabs:
                if fablabs[k].id == i["lab_id"]:
                    current_project.lab = fablabs[k]
        else:
            current_project.lab = None
        current_project.owner_id = i["owner_id"]
        current_project.created_at = i["created_at"]
        current_project.updated_at = i["updated_at"]
        current_project.vimeo = i["vimeo"]
        current_project.flickr = i["flickr"]
        current_project.youtube = i["youtube"]
        current_project.drive = i["drive"]
        current_project.twitter = i["twitter"]
        current_project.facebook = i["facebook"]
        current_project.googleplus = i["googleplus"]
        current_project.instagram = i["instagram"]
        current_project.status = i["status"]
        current_project.version = i["version"]
        current_project.faq = i["faq"]
        current_project.scope = i["scope"]
        current_project.community = i["community"]
        current_project.lookingfor = i["lookingfor"]
        current_project.cover = i["cover"]
        url = project_url + str(current_project.id)
        current_project.url = url
        # Add the project
        projects[current_project.id] = current_project

    # Return a dictiornary / json
    if format.lower() == "dict" or format.lower() == "json":
        output = {}
        for j in projects:
            project_dict = projects[j].__dict__
            # Convert the lab from a Fab Lab object to a dict
            if project_dict["lab"] is not None:
                project_dict["lab"] = project_dict["lab"].__dict__
            output[j] = project_dict
    # Return a geojson, only for projects linked to a lab
    elif format.lower() == "geojson" or format.lower() == "geo":
        projects_list = []
        for p in projects:
            if projects[p].lab_id is not None:
                single_project = projects[p].__dict__
                if projects[p].lab is not None:
                    single_project["lab"] = single_project["lab"].__dict__
                for l in fablabs:
                    single_lab = fablabs[l].__dict__
                    if single_lab["id"] == single_project["lab_id"]:
                        project_lab = Feature(
                            type="Feature",
                            geometry=Point((single_lab["latitude"],
                                            single_lab["longitude"])),
                            properties=single_project)
                        projects_list.append(project_lab)
                output = dumps(FeatureCollection(projects_list))
    # Return an object
    elif format.lower() == "object" or format.lower() == "obj":
        output = projects
    # Default: return an object
    else:
        output = projects
    # Return a proper json
    if format.lower() == "json":
        output = json.dumps(output)
    return output


def projects_count():
    """Gets the number of current projects submitted on fablabs.io."""

    projects = data_from_fablabs_io(fablabs_io_projects_api_url_v0)

    return len(projects["projects"])


if __name__ == "__main__":
    print get_labs(format="json")



# -*- encoding: utf-8 -*-
#
# Fuctions for the other modules
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#

# Import necessary modules

from geopy.geocoders import OpenCage
from time import sleep
from incf.countryutils import transformations


def get_location(query, format, api_key):
    """Get geographic data of a lab in a coherent way for all labs."""

    # Play nice with the API...
    sleep(1)
    geolocator = OpenCage(api_key=api_key, timeout=10)

    # Variables for storing the data
    data = {"city": None,
            "address_1": None,
            "postal_code": None,
            "country": None,
            "county": None,
            "state": None,
            "country_code": None,
            "latitude": None,
            "longitude": None,
            "continent": None}
    road = ""
    number = ""
    # Default None values
    location_data = {"city": None,
                     "road": None,
                     "house_number": None,
                     "postcode": None,
                     "country": None,
                     "county": None,
                     "state": None,
                     "ISO_3166-1_alpha-2": None,
                     "country_code": None,
                     "lat": None,
                     "lng": None}

    # Reverse geocoding ... from coordinates to address
    if format == "reverse":
        # If the query (coordinates) is not empty
        if query is None or len(query) < 3:
            pass
        else:
            location = geolocator.reverse(query)
            if location is not None:
                location_data = location[0].raw[u'components']
                location_data["lat"] = location[0].raw[u'geometry']["lat"]
                location_data["lng"] = location[0].raw[u'geometry']["lng"]
    # Direct geocoding ... from address to coordinates and full address
    if format == "direct":
        # If the query (address) is not empty
        if query is None or len(query) < 3:
            pass
        else:
            location = geolocator.geocode(query)
            if location is not None:
                location_data = location.raw[u'components']
                location_data["lat"] = location.raw[u'geometry']["lat"]
                location_data["lng"] = location.raw[u'geometry']["lng"]

    # Extract the meaningful data
    for component in location_data:
        if component == "town" or component == "city":
            data["city"] = location_data[component]
        if component == "road":
            road = location_data[component]
        if component == "house_number":
            number = location_data[component]
        if component == "postcode":
            data["postal_code"] = location_data[component]
        if component == "country":
            data["country"] = location_data[component]
        if component == "county":
            data["county"] = location_data[component]
        if component == "state":
            data["state"] = location_data[component]
        if component == "ISO_3166-1_alpha-2":
            data["country_code"] = location_data[component]
    # The address need to be reconstructed
    data["address_1"] = unicode(road) + " " + unicode(number)
    data["latitude"] = location_data["lat"]
    data["longitude"] = location_data["lng"]
    # Format the country code to three letters
    try:
        country_data = transformations.cca2_to_ccn(data["country_code"])
        data["country_code"] = transformations.ccn_to_cca3(country_data)
    except:
        data["country_code"] = None
    # Get the continent
    try:
        country_data = transformations.cc_to_cn(data["country_code"])
        data["continent"] = transformations.cn_to_ctn(country_data)
    except:
        data["continent"] = None

    # Return the final data
    return data


if __name__ == "__main__":
    pass



# -*- encoding: utf-8 -*-
#
# Access data from hackerspaces.org
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#


from classes import Lab
from utils import get_location

import json
from simplemediawiki import MediaWiki
import mwparserfromhell
import pandas as pd

hackerspaces_org_api_url = "https://wiki.hackerspaces.org/w/api.php"


class Hackerspace(Lab):

    """Represents a Hackerspace as it is described on hackerspaces.org."""

    def __init__(self):
        self.source = "hackerspaces.org"
        self.lab_type = "Hackerspace"


def get_single_lab(lab_slug, open_cage_api_key):
    """Gets data from a single lab from hackerspaces.org."""
    wiki = MediaWiki(hackerspaces_org_api_url)
    wiki_response = wiki.call(
        {'action': 'query',
         'titles': lab_slug,
         'prop': 'revisions',
         'rvprop': 'content'})

    # If we don't know the pageid...
    for i in wiki_response["query"]["pages"]:
        content = wiki_response["query"]["pages"][i]["revisions"][0]["*"]

    # Transform the data into a Lab object
    current_lab = Hackerspace()

    equipment_list = []

    # Parse the Mediawiki code
    wikicode = mwparserfromhell.parse(content)
    for k in wikicode.filter_templates():
        element_name = unicode(k.name)
        if "Hackerspace" in element_name:
            for j in k.params:
                current_lab.name = lab_slug
                j_value = unicode(j.value)
                j_name = unicode(j.name)
                # Remove new line in content
                if j_value[-1:] == "\n" or j_value[:1] == "\n":
                    j_value = j_value.replace('\n', '')
                if j_name == "logo":
                    current_lab.logo = j_value
                if j_name == "founding":
                    current_lab.founding = j_value
                if j_name == "coordinate":
                    # Clean the coordinates
                    j_value = j_value.replace('"', '')
                    j_value = j_value.replace('N', '')
                    j_value = j_value.replace('S', '')
                    j_value = j_value.replace('W', '')
                    j_value = j_value.replace('E', '')
                    j_value = j_value.replace(u'°', '')
                    j_value = j_value.replace(' ', '')
                    # Get the full address with the coordinates
                    address = get_location(query=j_value, format="reverse", api_key=open_cage_api_key)
                    current_lab.city = address["city"]
                    current_lab.county = address["county"]
                    current_lab.state = address["state"]
                    current_lab.postal_code = address["postal_code"]
                    current_lab.address_1 = address["address_1"]
                    current_lab.country = address["country"]
                    current_lab.country_code = address["country_code"]
                    current_lab.continent = address["continent"]
                    current_lab.latitude = address["latitude"]
                    current_lab.longitude = address["longitude"]
                if j_name == "membercount":
                    current_lab.membercount = j_value
                if j_name == "fee":
                    current_lab.fee = j_value
                if j_name == "size":
                    current_lab.size = j_value
                if j_name == "status":
                    current_lab.status = j_value
                if j_name == "site":
                    current_lab.site = j_value
                if j_name == "wiki":
                    current_lab.wiki = j_value
                if j_name == "irc":
                    current_lab.irc = j_value
                if j_name == "jabber":
                    current_lab.jabber = j_value
                if j_name == "phone":
                    current_lab.phone = j_value
                if j_name == "youtube":
                    current_lab.youtube = j_value
                if j_name == "eventbrite":
                    current_lab.eventbrite = j_value
                if j_name == "facebook":
                    current_lab.facebook = j_value
                if j_name == "ustream":
                    current_lab.ustream = j_value
                if j_name == "flickr":
                    current_lab.flickr = j_value
                if j_name == "twitter":
                    current_lab.twitter = j_value
                if j_name == "googleplus":
                    current_lab.googleplus = j_value
                if j_name == "email":
                    current_lab.email = j_value
                if j_name == "maillist":
                    current_lab.maillist = j_value
                if j_name == "ical":
                    current_lab.ical = j_value
                if j_name == "forum":
                    current_lab.forum = j_value
        elif "Equipment" in element_name:
            for j in k.params:
                equipment_list.append(j.replace("equipment=", ""))

            current_lab.equipment = equipment_list

    # Load the free text
    freetext = ""
    for k in wikicode._nodes:
        try:
            test_value = k.name
        except AttributeError:
            freetext += unicode(k)
    current_lab.text = freetext

    return current_lab


def get_labs(format, open_cage_api_key):
    """Gets data from all labs from hackerspaces.org."""

    labs = []

    # Get the first page of data
    wiki = MediaWiki(hackerspaces_org_api_url)
    wiki_response = wiki.call(
        {'action': 'query',
         'list': 'categorymembers',
         'cmtitle': 'Category:Hackerspace',
         'cmlimit': '500'})
    nextpage = wiki_response["query-continue"]["categorymembers"]["cmcontinue"]

    urls = []
    for i in wiki_response["query"]["categorymembers"]:
        urls.append(i["title"].replace(" ", "_"))

    # Load all the Labs in the first page
    for i in urls:
        current_lab = get_single_lab(i, open_cage_api_key)
        labs.append(current_lab)

    # Load all the Labs from the other pages
    while "query-continue" in wiki_response:
        wiki = MediaWiki(hackerspaces_org_api_url)
        wiki_response = wiki.call({'action': 'query',
                                   'list': 'categorymembers',
                                   'cmtitle': 'Category:Hackerspace',
                                   'cmlimit': '500',
                                   "cmcontinue": nextpage})

        urls = []
        for i in wiki_response["query"]["categorymembers"]:
            urls.append(i["title"].replace(" ", "_"))

        # Load all the Labs
        for i in urls:
            current_lab = get_single_lab(i, open_cage_api_key)
            labs.append(current_lab)

        if "query-continue" in wiki_response:
            nextpage = wiki_response[
                "query-continue"]["categorymembers"]["cmcontinue"]
        else:
            break

    # Transform the list into a dictionary
    labs_dict = {}
    for j, k in enumerate(labs):
        labs_dict[j] = k.__dict__

    # Return a dictiornary / json
    if format.lower() == "dict" or format.lower() == "json":
        output = labs_dict
    # Return a geojson
    elif format.lower() == "geojson" or format.lower() == "geo":
        labs_list = []
        for l in labs_dict:
            single = labs_dict[l].__dict__
            single_lab = Feature(
                type="Feature",
                geometry=Point((single["latitude"], single["longitude"])),
                properties=single)
            labs_list.append(single_lab)
        output = dumps(FeatureCollection(labs_list))
    # Return a Pandas DataFrame
    elif format.lower() == "pandas" or format.lower() == "dataframe":
        output = labs_dict
        # Transform the dict into a Pandas DataFrame
        output = pd.DataFrame.from_dict(output)
        output = output.transpose()
        output = output.set_index(['name'])
    # Return an object
    elif format.lower() == "object" or format.lower() == "obj":
        output = labs
    # Default: return an object
    else:
        output = labs
    # Return a proper json
    if format.lower() == "json":
        output = json.dumps(labs_dict)
    return output


def labs_count():
    """Gets the number of current Hackerspaces registered on hackerspaces.org."""

    labs = get_labs()

    return len(labs)


if __name__ == "__main__":
    pass



# -*- encoding: utf-8 -*-
#
# Classes
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#

class Lab(object):
    """Represents a Lab with the most common fields found."""

    def __init__(self):
        self.source = ""
        self.id = ""
        self.name = ""
        self.lab_type = ""
        self.continent = ""
        self.city = ""
        self.country_code = ""
        self.country = ""
        self.address_1 = ""
        self.address_2 = ""
        self.postal_code = ""
        self.county = ""
        self.state = ""
        self.latitude = ""
        self.longitude = ""
        self.url = ""
        self.slug = ""
        self.email = ""
        self.avatar = ""
        self.blurb = ""
        self.description = ""
        self.phone = ""
        self.capabilities = ""
        self.manager = ""
        self.founding = ""
        self.links = ""



# -*- encoding: utf-8 -*-
#
# Access data from repaircafe.org
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#


from classes import Lab

import json
from geojson import dumps, Feature, Point, FeatureCollection
from geopy.geocoders import Nominatim
import pycountry
from pycountry_convert import convert_country_alpha2_to_continent
from time import sleep
from selenium import webdriver
from bs4 import BeautifulSoup
import requests
import pandas as pd


# Geocoding variable
geolocator = Nominatim()

# Endpoints
fablabs_io_labs_api_url_v0 = "https://api.fablabs.io/v0/labs.json"
fablabs_io_projects_api_url_v0 = "https://api.fablabs.io/v0/projects.json"


class RepairCafe(Lab):
    """Represents a Repair Cafe as it is described on repaircafe.org."""

    def __init__(self):
        self.source = "repaircafe.org"
        self.lab_type = "Repair Cafe"


def data_from_repaircafe_org():
    """Gets data from repaircafe_org."""

    # Use Chrome as a browser
    # browser = webdriver.Chrome()
    # Use PhantomJS as a browser
    browser = webdriver.PhantomJS('phantomjs')
    browser.get("https://repaircafe.org/en/?s=Contact+the+local+organisers")
    browser.maximize_window()

    # Iterate over results (the #viewmore_link button)
    viewmore_button = True
    while viewmore_button:
        try:
            viewmore = browser.find_element_by_id("viewmore_link")
            # Scroll to the link in order to make it visible
            browser.execute_script("arguments[0].scrollIntoView();", viewmore)
            # Keep searching
            viewmore.click()
        except:
            # If there's an error, we have reached the end of the search
            viewmore_button = False
        # Give a bit of time for loading the search results
        sleep(2)

    # Load the source code
    page_source = BeautifulSoup(browser.page_source, "lxml")
    # Close the browser
    browser.quit()
    # Parse the source code in order to find all the links under H4s
    data = []
    for h4 in page_source.find_all("h4"):
        for a in h4.find_all('a', href=True):
            data.append({"name": a.contents[0], "url": a['href']})

    return data


def get_labs(format):
    """Gets Repair Cafe data from repairecafe.org."""

    data = data_from_repaircafe_org()

    repaircafes = {}

    # Load all the Repair Cafes
    for i in data:
        # Create a lab
        current_lab = RepairCafe()
        # Add existing data from first scraping
        current_lab.name = i["name"]
        slug = i["url"].replace("https://repaircafe.org/locations/", "")
        if slug.endswith("/"):
            slug.replace("/", "")
        current_lab.slug = slug
        current_lab.url = i["url"]
        # Scrape for more data
        page_request = requests.get(i["url"])
        if page_request.status_code == 200:
            page_source = BeautifulSoup(page_request.text, "lxml")
        else:
            output = "There was an error while accessing data on repaircafe.org."

        # Find Facebook and Twitter links, add also the other ones
        current_lab.links = {"facebook": "", "twitter": ""}
        column = page_source.find_all("div", class_="sc_column_item_2")
        for j in column:
            for p in j.find_all('p'):
                for a in p.find_all('a', href=True):
                    if "facebook" in a['href']:
                        current_lab.links["facebook"] = a['href']
                    elif "twitter" in a['href']:
                        current_lab.links["twitter"] = a['href']
                    else:
                        current_lab.links[a['href']] = a['href']

        # Find address
        column = page_source.find_all("div", class_="sc_column_item_1")
        for x in column:
            if x.string:
                print x.string.strip()

        exit()
        # current_lab.address_1 = i["address_1"]
        # current_lab.address_2 = i["address_2"]
        # current_lab.address_notes = i["address_notes"]
        # current_lab.blurb = i["blurb"]
        # current_lab.city = i["city"]
        # current_lab.country_code = i["country_code"]
        # current_lab.county = i["county"]
        # current_lab.description = i["description"]
        # current_lab.email = i["email"]
        # current_lab.id = i["id"]
        # current_lab.phone = i["phone"]
        # current_lab.postal_code = i["postal_code"]
        #
        #
        # current_lab.continent = convert_country_alpha2_to_continent(i[
        #     "country_code"].upper())
        # current_country = pycountry.countries.get(
        #     alpha_2=i["country_code"].upper())
        # current_lab.country_code = current_country.alpha_3
        # current_lab.country = current_country.name

        # if i["longitude"] is None or i["latitude"] is None:
        #     # Be nice with the geocoder API limit
        #     errorsb += 1
        #     # sleep(10)
        #     # location = geolocator.geocode(
        #     #     {"city": i["city"],
        #     #      "country": i["country_code"].upper()},
        #     #     addressdetails=True,
        #     #     language="en")
        #     # if location is not None:
        #     #     current_lab.latitude = location.latitude
        #     #     current_lab.longitude = location.longitude
        #     #     if "county" in location.raw["address"]:
        #     #         current_lab.county = location.raw["address"][
        #     #             "county"].encode('utf-8')
        #     #     if "state" in location.raw["address"]:
        #     #         current_lab.state = location.raw["address"][
        #     #             "state"].encode('utf-8')
        # else:
        #     # Be nice with the geocoder API limit
        #     sleep(10)
        #     errorsa += 1
        #     # location = geolocator.reverse((i["latitude"], i["longitude"]))
        #     # if location is not None:
        #     #     if "county" in location.raw["address"]:
        #     #         current_lab.county = location.raw["address"][
        #     #             "county"].encode('utf-8')
        #     #     if "state" in location.raw["address"]:
        #     #         current_lab.state = location.raw["address"][
        #     #             "state"].encode('utf-8')

        # Add the lab to the list
        repaircafes[slug] = current_lab
    # Return a dictiornary / json
    if format.lower() == "dict" or format.lower() == "json":
        output = {}
        for j in repaircafes:
            output[j] = repaircafes[j].__dict__
    # Return a geojson
    elif format.lower() == "geojson" or format.lower() == "geo":
        labs_list = []
        for l in repaircafes:
            single = repaircafes[l].__dict__
            single_lab = Feature(
                type="Feature",
                geometry=Point((single["latitude"], single["longitude"])),
                properties=single)
            labs_list.append(single_lab)
        output = dumps(FeatureCollection(labs_list))
    # Return a Pandas DataFrame
    elif format.lower() == "pandas" or format.lower() == "dataframe":
        output = {}
        for j in repaircafes:
            output[j] = repaircafes[j].__dict__
        # Transform the dict into a Pandas DataFrame
        output = pd.DataFrame.from_dict(output)
        output = output.transpose()
    # Return an object
    elif format.lower() == "object" or format.lower() == "obj":
        output = repaircafes
    # Default: return an oject
    else:
        output = repaircafes
    # Return a proper json
    if format.lower() == "json":
        output = json.dumps(output)
    return output


def labs_count():
    """Gets the number of current Repair Cafes registered on repaircafe.org."""

    repaircafes = data_from_repaircafe_org()

    return len(repaircafes["labs"])


if __name__ == "__main__":
    print get_labs(format="json")



#from distutils.core import setup
from setuptools import setup
setup(
    name='makerlabs',
    packages=['makerlabs'],
    install_requires=[
        "kitchen",
        "requests",
        "mwparserfromhell",
        "simplemediawiki",
        "wsgiref",
        "geojson",
        "geopy",
        "beautifulsoup4",
        "lxml",
        "pandas",
        "us",
        "pycountry",
        "pycountry_convert",
        "incf.countryutils",
        "selenium"
    ],
    version='0.21.2',
    description='A python library for accessing online data about Makerspaces, Fab Labs, Hackerspaces, TechShop...',
    author='Massimo Menichinelli',
    author_email='info@openp2pdesign.org',
    url='https://github.com/openp2pdesign/makerlabs',
    download_url='https://github.com/openp2pdesign/makerlabs/releases/tag/v0.21.2',
    keywords=['Fab Lab', 'Fab Lab', 'Makerspace', 'Hackerspace', 'TechShop',
              'Makers'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "Environment :: Web Environment",
        "Operating System :: OS Independent",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
    ], )



# -*- encoding: utf-8 -*-
#
# Rebuild a timeline of makerlabs
#
# Author: Massimo Menichinelli
# Homepage: http://www.openp2pdesign.org
# License: LGPL v.3
#
#

# Import all the mapped platforms
import diybio_org
import fablabs_io
import makeinitaly_foundation
import hackaday_io
import hackerspaces_org
import makery_info
import nesta
import techshop_ws

import pandas as pd


def get_multiple_data():
    """Get data from all the platforms listed in makerlabs."""

    # Get data from all the mapped platforms
    all_labs = {}
    all_labs["diybio_org"] = diybio_org.get_labs(format="dict")
    all_labs["fablabs_io"] = fablabs_io.get_labs(format="dict")
    all_labs["makeinitaly_foundation"] = makeinitaly_foundation.get_labs(
        format="dict")
    all_labs["hackaday_io"] = hackaday_io.get_labs(format="dict")
    all_labs["hackerspaces_org"] = hackerspaces_org.get_labs(format="dict")
    all_labs["makery_info"] = makery_info.get_labs(format="dict")
    all_labs["nesta"] = nesta.get_labs(format="dict")
    # all_labs["techshop_ws"] = techshop_ws.get_labs(format="dict")

    return all_labs


def get_timeline(source):
    """Rebuild a timeline of the history of makerlabs."""

    # Set up the pandas timeseries dataframe
    timeline_format = ["name", "type", "source", "country", "city", "latitude",
                       "longitude", "website_url", "twitter_url",
                       "facebook_page_url", "facebook_group_url",
                       "whois_start", "whois_end", "wayback_start",
                       "wayback_end", "twitter_start", "twitter_end",
                       "facebook_start", "facebook_end"]
    timeline = pd.DataFrame(timeline_format)

    # Getdata from all the mapped platforms
    if source.lower() == "diybio.org":
        data = diybio_org.get_labs(format="dict")
    elif source.lower() == "fablabs_io":
        data = fablabs_io.get_labs(format="dict")
    elif source.lower() == "makeinitaly_foundation":
        data = makeinitaly_foundation.get_labs(format="dict")
    elif source.lower() == "hackaday_io":
        data = hackaday_io.get_labs(format="dict")
    elif source.lower() == "hackerspaces_org":
        data = hackerspaces_org.get_labs(format="dict")
    elif source.lower() == "makery_info":
        data = makery_info.get_labs(format="dict")
    elif source.lower() == "nesta":
        data = nesta.get_labs(format="dict")
    elif source.lower() == "all":
        pass

    # Fill the dataframe with basic details
    for lab in labs_data:
        for link in lab.links:
            print link
            if "twitter" in link:
                print link
            if "facebook" in link:
                print link
        lab_dataframe_dict = {"name": lab.name,
                              "type": lab.lab_type,
                              "source": lab.source,
                              "country": lab.country,
                              "city": lab.city,
                              "latitude": lab.latitude,
                              "longitude": lab.longitude,
                              "website_url": lab.url}
        timeline.append(lab_dataframe_dict)
        ["name", "type", "source", "country", "city", "lat", "long",
         "website_url", "twitter_url", "facebook_page_url",
         "facebook_group_url", "whois_start", "whois_end", "wayback_start",
         "wayback_end", "twitter_start", "twitter_end", "facebook_start",
         "facebook_end"]

    # Get time data from platforms, whenever possible

    # Get domain data (whois)

    # Get subdomain data (Internet Archive)

    # Get social media data (Twitter)

    # Get social media data (Facebook)

    return timeline


if __name__ == "__main__":
    print get_timeline("fablabs_io")
